@@ -30,7 +30,6 @@
 #include <gtk/gtkvbbox.h>
 #include <libgnomedb/libgnomedb.h>
 #include <libgnomedb/gnome-db-util.h>
-#include <libgnomeui/gnome-window-icon.h>
 #include "dsn-config.h"
 #include "dsn-properties-dialog.h"
 
@@ -65,7 +64,7 @@ show_data_sources (DsnConfigPrivate *priv)
 	GtkTreeViewColumn *col;
 
 	/* create the data model */
-	model = gda_data_model_array_new (3);
+	model = gda_data_model_array_new_with_g_types (3, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_BOOLEAN);
 	gda_data_model_set_column_title (model, 0, _("Name"));
 	gda_data_model_set_column_title (model, 1, _("Description"));
 	gda_data_model_set_column_title (model, 2, _("Global"));
@@ -310,6 +309,7 @@ foreach_remove_cb (GtkTreeModel *tree_model, GtkTreePath *path, GtkTreeIter *ite
 					      GTK_DIALOG_MODAL,
 					      GTK_STOCK_YES, GTK_RESPONSE_YES,
 					      GTK_STOCK_NO, GTK_RESPONSE_NO, NULL);
+	gtk_dialog_set_has_separator (GTK_DIALOG (dialog), FALSE);
 	str1 = g_strdup_printf (_("Are you sure you want to remove the data source '%s'?"), dsn);
 	str = g_strdup_printf ("<b>Data source removal confirmation:</b>\n\n%s", str1);
 	g_free (str1);
