@@ -92,6 +92,7 @@ provider_config_new (void)
 	GtkWidget *box;
 	GtkWidget *button;
 	GtkWidget *label;
+	GtkWidget *sw;
 	gchar *title;
 
 	priv = g_new0 (ProviderConfigPrivate, 1);
@@ -117,15 +118,20 @@ provider_config_new (void)
 	gtk_widget_show (priv->title);
 
 	/* create the provider list */
-	priv->provider_list = gnome_db_raw_grid_new (NULL);
-	gnome_db_data_widget_column_set_editable (GNOME_DB_DATA_WIDGET (priv->provider_list), 0, FALSE);
-	g_object_set (G_OBJECT (priv->provider_list), "info_cell_visible", FALSE, NULL);
-
-	gtk_widget_show (priv->provider_list);
-	gtk_table_attach (GTK_TABLE (table), priv->provider_list, 0, 1, 1, 2,
+	sw = gtk_scrolled_window_new (NULL, NULL);
+	gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (sw), GTK_POLICY_AUTOMATIC,
+					GTK_POLICY_AUTOMATIC);	
+	gtk_table_attach (GTK_TABLE (table), sw, 0, 1, 1, 2,
 			  GTK_FILL | GTK_SHRINK | GTK_EXPAND,
 			  GTK_FILL | GTK_SHRINK | GTK_EXPAND,
 			  0, 0);
+
+	priv->provider_list = gnome_db_raw_grid_new (NULL);
+	gnome_db_data_widget_column_set_editable (GNOME_DB_DATA_WIDGET (priv->provider_list), 0, FALSE);
+	g_object_set (G_OBJECT (priv->provider_list), "info_cell_visible", FALSE, NULL);
+	gtk_container_add (GTK_CONTAINER (sw), priv->provider_list);
+	
+	gtk_widget_show_all (sw);
 	show_providers (priv);
 
 	/* add tip */
