@@ -1,6 +1,6 @@
 /* gnome-db-entry-combo.c
  *
- * Copyright (C) 2003 - 2005 Vivien Malerba
+ * Copyright (C) 2003 - 2006 Vivien Malerba
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -43,16 +43,17 @@ static void          gnome_db_entry_combo_emit_signal (GnomeDbEntryCombo *combo)
 static void          real_combo_block_signals (GnomeDbEntryCombo *wid);
 static void          real_combo_unblock_signals (GnomeDbEntryCombo *wid);
 
-/* GnomeDbDataEntry interface (value must be a GDA_VALUE_TYPE_LIST) */
+/* GnomeDbDataEntry interface (value must be a GDA_TYPE_LIST) */
 static void            gnome_db_entry_combo_data_entry_init   (GnomeDbDataEntryIface *iface);
-static void            gnome_db_entry_combo_set_value         (GnomeDbDataEntry *de, const GdaValue * value);
-static GdaValue       *gnome_db_entry_combo_get_value         (GnomeDbDataEntry *de);
-static void            gnome_db_entry_combo_set_value_orig    (GnomeDbDataEntry *de, const GdaValue * value);
-static const GdaValue *gnome_db_entry_combo_get_value_orig    (GnomeDbDataEntry *de);
-static void            gnome_db_entry_combo_set_value_default (GnomeDbDataEntry *de, const GdaValue * value);
+static void            gnome_db_entry_combo_set_value         (GnomeDbDataEntry *de, const GValue * value);
+static GValue         *gnome_db_entry_combo_get_value         (GnomeDbDataEntry *de);
+static void            gnome_db_entry_combo_set_value_orig    (GnomeDbDataEntry *de, const GValue * value);
+static const GValue   *gnome_db_entry_combo_get_value_orig    (GnomeDbDataEntry *de);
+static void            gnome_db_entry_combo_set_value_default (GnomeDbDataEntry *de, const GValue * value);
 static void            gnome_db_entry_combo_set_attributes    (GnomeDbDataEntry *de, guint attrs, guint mask);
 static guint           gnome_db_entry_combo_get_attributes    (GnomeDbDataEntry *de);
 static gboolean        gnome_db_entry_combo_expand_in_layout  (GnomeDbDataEntry *de);
+static void            gnome_db_entry_combo_grab_focus        (GnomeDbDataEntry *de);
 
 /* properties */
 enum
@@ -66,9 +67,9 @@ enum
  */
 typedef struct {
 	GdaParameterListNode *node;
-	const GdaValue       *value;    /* we don't own the value, since it belongs to a GdaDataModel => don't free it */
-	GdaValue             *value_orig;
-	GdaValue             *value_default;
+	const GValue       *value;    /* we don't own the value, since it belongs to a GdaDataModel => don't free it */
+	GValue             *value_orig;
+	GValue             *value_default;
 } ComboNode;
 #define COMBO_NODE(x) ((ComboNode*)(x))
 
@@ -137,6 +138,7 @@ gnome_db_entry_combo_data_entry_init (GnomeDbDataEntryIface *iface)
         iface->get_attributes = gnome_db_entry_combo_get_attributes;
         iface->get_handler = NULL;
         iface->expand_in_layout = gnome_db_entry_combo_expand_in_layout;
+	iface->grab_focus = gnome_db_entry_combo_grab_focus;
 }
 
 
@@ -155,6 +157,12 @@ gnome_db_entry_combo_class_init (GnomeDbEntryComboClass *class)
         g_object_class_install_property (object_class, PROP_SET_DEFAULT_IF_INVALID,
 					 g_param_spec_boolean ("set_default_if_invalid", NULL, NULL, FALSE,
                                                                (G_PARAM_READABLE | G_PARAM_WRITABLE)));
+
+	/* RC setting */
+#define RC_STRING  \
+	"style \"gnomedb\" { GnomeDbCombo::appears-as-list = 1 }" \
+	"class \"GtkComboBox\" style \"gnomedb\""
+	/*gtk_rc_parse_string (RC_STRING);*/
 }
 
 static void
@@ -258,7 +266,7 @@ gnome_db_entry_combo_new (GdaParameterList *paramlist, GdaParameterListSource *s
 		cnode->value = gda_parameter_get_value (cnode->node->param);
 		combo->priv->combo_nodes = g_slist_append (combo->priv->combo_nodes, cnode);
 
-		values = g_slist_append (values, (GdaValue *) cnode->value);
+		values = g_slist_append (values, (GValue *) cnode->value);
 		if (gda_parameter_get_not_null (cnode->node->param))
 			null_possible = FALSE;
 
@@ -300,7 +308,7 @@ gnome_db_entry_combo_dispose (GObject *object)
 	GnomeDbEntryCombo *combo;
 
 	g_return_if_fail (object != NULL);
-	g_return_if_fail (IS_GNOME_DB_ENTRY_COMBO (object));
+	g_return_if_fail (GNOME_DB_IS_ENTRY_COMBO (object));
 
 	combo = GNOME_DB_ENTRY_COMBO (object);
 
@@ -412,12 +420,12 @@ combo_contents_changed_cb (GnomeDbCombo *entry, GnomeDbEntryCombo *combo)
 /**
  * gnome_db_entry_combo_set_values
  * @combo: a #GnomeDbEntryCombo widet
- * @values: a list of #GdaValue values, or %NULL
+ * @values: a list of #GValue values, or %NULL
  *
  * Sets the values of @combo to the specified ones. None of the
  * values provided in the list is modified.
  *
- * @values holds a list of #GdaValue values, one for each parameter that is present in the @node argument
+ * @values holds a list of #GValue values, one for each parameter that is present in the @node argument
  * of the gnome_db_entry_combo_new() function which created @combo.
  *
  * An error can occur when there is no corresponding value(s) to be displayed
@@ -425,7 +433,7 @@ combo_contents_changed_cb (GnomeDbCombo *entry, GnomeDbEntryCombo *combo)
  *
  * If @values is %NULL, then the entry itself is set to NULL;
  *
- * Returns: TRUE if no error occured.
+ * Returns: TRUE if no error occurred.
  */
 gboolean
 gnome_db_entry_combo_set_values (GnomeDbEntryCombo *combo, GSList *values)
@@ -434,13 +442,13 @@ gnome_db_entry_combo_set_values (GnomeDbEntryCombo *combo, GSList *values)
 	gboolean allnull = TRUE;
 	GSList *list;
 
-	g_return_val_if_fail (combo && IS_GNOME_DB_ENTRY_COMBO (combo), FALSE);
+	g_return_val_if_fail (combo && GNOME_DB_IS_ENTRY_COMBO (combo), FALSE);
 	g_return_val_if_fail (combo->priv, FALSE);
 
-	/* try to determine if all the values are NULL or of type GDA_VALUE_TYPE_NULL */
+	/* try to determine if all the values are NULL or of type GDA_TYPE_NULL */
 	list = values;
 	while (list && allnull) {
-		if (list->data && (gda_value_get_type ((GdaValue *)list->data) != GDA_VALUE_TYPE_NULL))
+		if (list->data && (G_VALUE_TYPE ((GValue *)list->data) != GDA_TYPE_NULL))
 			allnull = FALSE;
 		list = g_slist_next (list);
 	}
@@ -478,20 +486,18 @@ gnome_db_entry_combo_set_values (GnomeDbEntryCombo *combo, GSList *values)
 			err = TRUE;
 	}
 	else  { /* set to NULL */
-		if (combo->priv->null_possible) {
-			GSList *list;
+		GSList *list;
+		list = combo->priv->combo_nodes;
+		while (list) {
+			COMBO_NODE (list->data)->value = NULL;
+			list = g_slist_next (list);
+		}
 
+		if (combo->priv->null_possible) {
 			real_combo_block_signals (combo);
 			gtk_combo_box_set_active (GTK_COMBO_BOX (combo->priv->combo_entry), -1);
 			real_combo_unblock_signals (combo);
 
-			/* adjusting the values */
-			list = combo->priv->combo_nodes;
-			while (list) {
-				COMBO_NODE (list->data)->value = NULL;
-				list = g_slist_next (list);
-			}
-
 			combo->priv->null_forced = TRUE;
 			combo->priv->default_forced = FALSE;
 		}
@@ -524,14 +530,14 @@ gnome_db_entry_combo_get_values (GnomeDbEntryCombo *combo)
 {
 	GSList *list;
 	GSList *retval = NULL;
-	g_return_val_if_fail (combo && IS_GNOME_DB_ENTRY_COMBO (combo), NULL);
+	g_return_val_if_fail (combo && GNOME_DB_IS_ENTRY_COMBO (combo), NULL);
 	g_return_val_if_fail (combo->priv, NULL);
 
 	list = combo->priv->combo_nodes;
 	while (list) {
 		ComboNode *node = COMBO_NODE (list->data);
 		
-		retval = g_slist_append (retval, (GdaValue *) node->value);
+		retval = g_slist_append (retval, (GValue *) node->value);
 
 		list = g_slist_next (list);
 	}
@@ -551,7 +557,7 @@ gnome_db_entry_combo_get_values (GnomeDbEntryCombo *combo)
 GSList *
 gnome_db_entry_combo_get_all_values (GnomeDbEntryCombo *combo)
 {
-	g_return_val_if_fail (combo && IS_GNOME_DB_ENTRY_COMBO (combo), NULL);
+	g_return_val_if_fail (combo && GNOME_DB_IS_ENTRY_COMBO (combo), NULL);
 	g_return_val_if_fail (combo->priv, NULL);
 
 	return gnome_db_combo_get_values_ext (combo->priv->combo_entry, 0, NULL);
@@ -560,7 +566,7 @@ gnome_db_entry_combo_get_all_values (GnomeDbEntryCombo *combo)
 /**
  * gnome_db_entry_combo_set_values_orig
  * @combo: a #GnomeDbEntryCombo widet
- * @values: a list of #GdaValue values
+ * @values: a list of #GValue values
  *
  * Sets the original values of @combo to the specified ones. None of the
  * values provided in the list is modified.
@@ -570,7 +576,7 @@ gnome_db_entry_combo_set_values_orig (GnomeDbEntryCombo *combo, GSList *values)
 {
 	GSList *list;
 
-	g_return_if_fail (combo && IS_GNOME_DB_ENTRY_COMBO (combo));
+	g_return_if_fail (combo && GNOME_DB_IS_ENTRY_COMBO (combo));
 	g_return_if_fail (combo->priv);
 
 	/* rem: we don't care if the values can be set or not, we just try */
@@ -591,7 +597,7 @@ gnome_db_entry_combo_set_values_orig (GnomeDbEntryCombo *combo, GSList *values)
 	if (values) {
 		GSList *nodes;
 		GSList *argptr;
-		const GdaValue *arg_value;
+		const GValue *arg_value;
 		gboolean equal = TRUE;
 		
 		g_return_if_fail (g_slist_length (values) == g_slist_length (combo->priv->combo_nodes));
@@ -603,15 +609,15 @@ gnome_db_entry_combo_set_values_orig (GnomeDbEntryCombo *combo, GSList *values)
 		argptr = values;
 		while (argptr && nodes && equal) {
 			GdaColumn *attrs;
-			GdaValueType type=GDA_VALUE_TYPE_NULL;
+			GType type=GDA_TYPE_NULL;
 			
 			attrs = gda_data_model_describe_column (combo->priv->source->data_model, 
 								COMBO_NODE (nodes->data)->node->source_column);
-			arg_value = (GdaValue*) (argptr->data);
+			arg_value = (GValue*) (argptr->data);
 			
 			if (arg_value)
-				type = gda_value_get_type ((GdaValue*) arg_value);
-			equal = (type == gda_column_get_gda_type (attrs));
+				type = G_VALUE_TYPE ((GValue*) arg_value);
+			equal = (type == gda_column_get_g_type (attrs));
 			
 			nodes = g_slist_next (nodes);
 			argptr = g_slist_next (argptr);
@@ -625,7 +631,7 @@ gnome_db_entry_combo_set_values_orig (GnomeDbEntryCombo *combo, GSList *values)
 			argptr = values;
 			while (argptr && nodes && equal) {
 				if (argptr->data)
-					COMBO_NODE (nodes->data)->value_orig = gda_value_copy ((GdaValue*) (argptr->data));
+					COMBO_NODE (nodes->data)->value_orig = gda_value_copy ((GValue*) (argptr->data));
 				nodes = g_slist_next (nodes);
 				argptr = g_slist_next (argptr);
 			}
@@ -649,7 +655,7 @@ gnome_db_entry_combo_get_values_orig (GnomeDbEntryCombo *combo)
 	GSList *retval = NULL;
 	gboolean allnull = TRUE;
 
-	g_return_val_if_fail (combo && IS_GNOME_DB_ENTRY_COMBO (combo), NULL);
+	g_return_val_if_fail (combo && GNOME_DB_IS_ENTRY_COMBO (combo), NULL);
 	g_return_val_if_fail (combo->priv, NULL);
 
 	list = combo->priv->combo_nodes;
@@ -657,7 +663,7 @@ gnome_db_entry_combo_get_values_orig (GnomeDbEntryCombo *combo)
 		ComboNode *node = COMBO_NODE (list->data);
 
 		if (node->value_orig && 
-		    (gda_value_get_type (node->value_orig) != GDA_VALUE_TYPE_NULL))
+		    (G_VALUE_TYPE (node->value_orig) != GDA_TYPE_NULL))
 			allnull = FALSE;
 		
 		retval = g_slist_append (retval, node->value_orig);
@@ -676,7 +682,7 @@ gnome_db_entry_combo_get_values_orig (GnomeDbEntryCombo *combo)
 /**
  * gnome_db_entry_combo_set_values_default
  * @combo: a #GnomeDbEntryCombo widet
- * @values: a list of #GdaValue values
+ * @values: a list of #GValue values
  *
  * Sets the default values of @combo to the specified ones. None of the
  * values provided in the list is modified.
@@ -684,7 +690,7 @@ gnome_db_entry_combo_get_values_orig (GnomeDbEntryCombo *combo)
 void
 gnome_db_entry_combo_set_values_default (GnomeDbEntryCombo *combo, GSList *values)
 {
-	g_return_if_fail (combo && IS_GNOME_DB_ENTRY_COMBO (combo));
+	g_return_if_fail (combo && GNOME_DB_IS_ENTRY_COMBO (combo));
 	g_return_if_fail (combo->priv);
 	TO_IMPLEMENT;
 }
@@ -695,26 +701,26 @@ gnome_db_entry_combo_set_values_default (GnomeDbEntryCombo *combo, GSList *value
  */
 
 static void
-gnome_db_entry_combo_set_value (GnomeDbDataEntry *iface, const GdaValue *value)
+gnome_db_entry_combo_set_value (GnomeDbDataEntry *iface, const GValue *value)
 {
 	GnomeDbEntryCombo *combo;
 
-        g_return_if_fail (iface && IS_GNOME_DB_ENTRY_COMBO (iface));
+        g_return_if_fail (iface && GNOME_DB_IS_ENTRY_COMBO (iface));
         combo = GNOME_DB_ENTRY_COMBO (iface);
         g_return_if_fail (combo->priv);
         g_return_if_fail (!value ||
-                          (value && (gda_value_isa ((GdaValue*) value, GDA_VALUE_TYPE_LIST) ||
-                                     gda_value_isa ((GdaValue*) value, GDA_VALUE_TYPE_LIST))));
+                          (value && (gda_value_isa ((GValue*) value, GDA_TYPE_LIST) ||
+                                     gda_value_isa ((GValue*) value, GDA_TYPE_LIST))));
 
 	TO_IMPLEMENT;
 }
 
-static GdaValue *
+static GValue *
 gnome_db_entry_combo_get_value (GnomeDbDataEntry *iface)
 {
         GnomeDbEntryCombo *combo;
 	
-        g_return_val_if_fail (iface && IS_GNOME_DB_ENTRY_COMBO (iface), NULL);
+        g_return_val_if_fail (iface && GNOME_DB_IS_ENTRY_COMBO (iface), NULL);
         combo = GNOME_DB_ENTRY_COMBO (iface);
         g_return_val_if_fail (combo->priv, NULL);
 	
@@ -724,23 +730,23 @@ gnome_db_entry_combo_get_value (GnomeDbDataEntry *iface)
 }
 
 static void
-gnome_db_entry_combo_set_value_orig (GnomeDbDataEntry *iface, const GdaValue * value)
+gnome_db_entry_combo_set_value_orig (GnomeDbDataEntry *iface, const GValue * value)
 {
         GnomeDbEntryCombo *combo;
 	
-        g_return_if_fail (iface && IS_GNOME_DB_ENTRY_COMBO (iface));
+        g_return_if_fail (iface && GNOME_DB_IS_ENTRY_COMBO (iface));
         combo = GNOME_DB_ENTRY_COMBO (iface);
         g_return_if_fail (combo->priv);
 
 	TO_IMPLEMENT;
 }
 
-static const GdaValue *
+static const GValue *
 gnome_db_entry_combo_get_value_orig (GnomeDbDataEntry *iface)
 {
         GnomeDbEntryCombo *combo;
 	
-        g_return_val_if_fail (iface && IS_GNOME_DB_ENTRY_COMBO (iface), NULL);
+        g_return_val_if_fail (iface && GNOME_DB_IS_ENTRY_COMBO (iface), NULL);
         combo = GNOME_DB_ENTRY_COMBO (iface);
         g_return_val_if_fail (combo->priv, NULL);
                      
@@ -750,11 +756,11 @@ gnome_db_entry_combo_get_value_orig (GnomeDbDataEntry *iface)
 }
 
 static void
-gnome_db_entry_combo_set_value_default (GnomeDbDataEntry *iface, const GdaValue * value)
+gnome_db_entry_combo_set_value_default (GnomeDbDataEntry *iface, const GValue * value)
 {
         GnomeDbEntryCombo *combo;
 	
-        g_return_if_fail (iface && IS_GNOME_DB_ENTRY_COMBO (iface));
+        g_return_if_fail (iface && GNOME_DB_IS_ENTRY_COMBO (iface));
         combo = GNOME_DB_ENTRY_COMBO (iface);
         g_return_if_fail (combo->priv);
 	
@@ -767,7 +773,7 @@ gnome_db_entry_combo_set_attributes (GnomeDbDataEntry *iface, guint attrs, guint
 {
 	GnomeDbEntryCombo *combo;
 
-	g_return_if_fail (iface && IS_GNOME_DB_ENTRY_COMBO (iface));
+	g_return_if_fail (iface && GNOME_DB_IS_ENTRY_COMBO (iface));
 	combo = GNOME_DB_ENTRY_COMBO (iface);
 	g_return_if_fail (combo->priv);
 
@@ -786,8 +792,8 @@ gnome_db_entry_combo_set_attributes (GnomeDbDataEntry *iface, guint attrs, guint
 				list = combo->priv->combo_nodes;
 				while (list && allnull) {
 					if (COMBO_NODE (list->data)->value_default && 
-					    (gda_value_get_type (COMBO_NODE (list->data)->value_default) != 
-					     GDA_VALUE_TYPE_NULL))
+					    (G_VALUE_TYPE (COMBO_NODE (list->data)->value_default) != 
+					     GDA_TYPE_NULL))
 						allnull = FALSE;
 					list = g_slist_next (list);
 				}
@@ -838,8 +844,8 @@ gnome_db_entry_combo_set_attributes (GnomeDbDataEntry *iface, guint attrs, guint
 				list = combo->priv->combo_nodes;
 				while (list && allnull) {
 					if (COMBO_NODE (list->data)->value_default && 
-					    (gda_value_get_type (COMBO_NODE (list->data)->value_default) != 
-					     GDA_VALUE_TYPE_NULL))
+					    (G_VALUE_TYPE (COMBO_NODE (list->data)->value_default) != 
+					     GDA_TYPE_NULL))
 						allnull = FALSE;
 					list = g_slist_next (list);
 				}
@@ -914,7 +920,7 @@ gnome_db_entry_combo_get_attributes (GnomeDbDataEntry *iface)
 	gboolean isunchanged = TRUE;
 	gboolean orig_value_exists = FALSE;
 
-	g_return_val_if_fail (iface && IS_GNOME_DB_ENTRY_COMBO (iface), 0);
+	g_return_val_if_fail (iface && GNOME_DB_IS_ENTRY_COMBO (iface), 0);
 	combo = GNOME_DB_ENTRY_COMBO (iface);
 	g_return_val_if_fail (combo->priv, 0);
 
@@ -924,7 +930,7 @@ gnome_db_entry_combo_get_attributes (GnomeDbDataEntry *iface)
 
 		/* NULL? */
 		if (COMBO_NODE (list->data)->value &&
-		    (gda_value_get_type ((GdaValue *) COMBO_NODE (list->data)->value) != GDA_VALUE_TYPE_NULL))
+		    (G_VALUE_TYPE ((GValue *) COMBO_NODE (list->data)->value) != GDA_TYPE_NULL))
 			isnull = FALSE;
 		
 		/* is unchanged */
@@ -932,13 +938,13 @@ gnome_db_entry_combo_get_attributes (GnomeDbDataEntry *iface)
 			orig_value_exists = TRUE;
 			
 			if (COMBO_NODE (list->data)->value && 
-			    (gda_value_get_type ((GdaValue *) COMBO_NODE (list->data)->value) == 
-			     gda_value_get_type (COMBO_NODE (list->data)->value_orig))) {
-				if (gda_value_get_type ((GdaValue *) COMBO_NODE (list->data)->value) == 
-				    GDA_VALUE_TYPE_NULL) 
+			    (G_VALUE_TYPE ((GValue *) COMBO_NODE (list->data)->value) == 
+			     G_VALUE_TYPE (COMBO_NODE (list->data)->value_orig))) {
+				if (G_VALUE_TYPE ((GValue *) COMBO_NODE (list->data)->value) == 
+				    GDA_TYPE_NULL) 
 					changed = FALSE;
 				else {
-					if (gda_value_compare ((GdaValue *) COMBO_NODE (list->data)->value, 
+					if (gda_value_compare ((GValue *) COMBO_NODE (list->data)->value, 
 							       COMBO_NODE (list->data)->value_orig))
 						changed = TRUE;
 				}
@@ -989,7 +995,7 @@ gnome_db_entry_combo_get_attributes (GnomeDbDataEntry *iface)
 			ComboNode *node = COMBO_NODE (nodes->data);
 
 			/* all the nodes are NULL ? */
-			if (node->value && (gda_value_get_type ((GdaValue *) node->value) != GDA_VALUE_TYPE_NULL))
+			if (node->value && (G_VALUE_TYPE ((GValue *) node->value) != GDA_TYPE_NULL))
 				allnull = FALSE;
 
 			nodes = g_slist_next (nodes);
@@ -1016,9 +1022,22 @@ gnome_db_entry_combo_expand_in_layout (GnomeDbDataEntry *iface)
 {
 	GnomeDbEntryCombo *combo;
 
-	g_return_val_if_fail (iface && IS_GNOME_DB_ENTRY_COMBO (iface), FALSE);
+	g_return_val_if_fail (iface && GNOME_DB_IS_ENTRY_COMBO (iface), FALSE);
 	combo = GNOME_DB_ENTRY_COMBO (iface);
 	g_return_val_if_fail (combo->priv, FALSE);
 
 	return FALSE;
 }
+
+static void
+gnome_db_entry_combo_grab_focus (GnomeDbDataEntry *iface)
+{
+	GnomeDbEntryCombo *combo;
+
+	g_return_if_fail (iface && GNOME_DB_IS_ENTRY_COMBO (iface));
+	combo = GNOME_DB_ENTRY_COMBO (iface);
+	g_return_if_fail (combo->priv);
+
+	if (combo->priv->combo_entry)
+		gtk_widget_grab_focus (combo->priv->combo_entry);
+}
