@@ -36,6 +36,7 @@ static void gnome_db_entry_wrapper_get_property (GObject *object,
 						 GParamSpec *pspec);
 
 static void contents_changed_cb (GtkWidget *entry, GnomeDbEntryWrapper *mgwrap);
+static void contents_activated_cb (GtkWidget *entry, GnomeDbEntryWrapper *mgwrap);
 static void check_correct_init (GnomeDbEntryWrapper *wid);
 static void block_signals (GnomeDbEntryWrapper *wid);
 static void unblock_signals (GnomeDbEntryWrapper *wid);
@@ -54,7 +55,7 @@ static guint           gnome_db_entry_wrapper_get_attributes    (GnomeDbDataEntr
 static GdaDataHandler *gnome_db_entry_wrapper_get_handler       (GnomeDbDataEntry *de);
 static gboolean        gnome_db_entry_wrapper_expand_in_layout  (GnomeDbDataEntry *de);
 static void            gnome_db_entry_wrapper_set_editable      (GnomeDbDataEntry *de, gboolean editable);
-
+static void            gnome_db_entry_wrapper_grab_focus        (GnomeDbDataEntry *de);
 
 /* properties */
 enum
@@ -79,6 +80,7 @@ struct  _GnomeDbEntryWrapperPriv {
 	gboolean                  null_possible;
 	gboolean                  default_possible;
 	gboolean                  show_actions;
+	gboolean                  editable;
 
 	/* property */
 	gboolean                  set_default_if_invalid;
@@ -132,6 +134,7 @@ gnome_db_entry_wrapper_data_entry_init (GnomeDbDataEntryIface *iface)
 	iface->get_handler = gnome_db_entry_wrapper_get_handler;
 	iface->expand_in_layout = gnome_db_entry_wrapper_expand_in_layout;
 	iface->set_editable = gnome_db_entry_wrapper_set_editable;
+	iface->grab_focus = gnome_db_entry_wrapper_grab_focus;
 }
 
 
@@ -201,7 +204,8 @@ check_correct_init (GnomeDbEntryWrapper *mgwrap)
 			gtk_widget_show (entry);
 			mgwrap->priv->entry = entry;
 			
-			(*mgwrap->priv->real_class->connect_signals) (mgwrap, G_CALLBACK (contents_changed_cb));
+			(*mgwrap->priv->real_class->connect_signals) (mgwrap, G_CALLBACK (contents_changed_cb),
+								      G_CALLBACK (contents_activated_cb));
 		}
 		else {
 			/* we need to exit because the program WILL BE unstable and WILL crash */
@@ -243,6 +247,7 @@ gnome_db_entry_wrapper_init (GnomeDbEntryWrapper *mgwrap)
 	mgwrap->priv->null_possible = TRUE;
 	mgwrap->priv->default_possible = FALSE;
 	mgwrap->priv->show_actions = TRUE;
+	mgwrap->priv->editable = TRUE;
 
 	mgwrap->priv->set_default_if_invalid = FALSE;
 }
@@ -281,28 +286,28 @@ gnome_db_entry_wrapper_set_property (GObject *object,
 	GnomeDbEntryWrapper *mgwrap = GNOME_DB_ENTRY_WRAPPER (object);
 	if (mgwrap->priv) {
 		switch (param_id) {
-		case PROP_SET_DEFAULT_IF_INVALID:
-			if (mgwrap->priv->set_default_if_invalid != g_value_get_boolean (value)) {
-				guint attrs;
-
-				mgwrap->priv->set_default_if_invalid = g_value_get_boolean (value);
-				attrs = gnome_db_data_entry_get_attributes (GNOME_DB_DATA_ENTRY (mgwrap));
-
-				if (mgwrap->priv->set_default_if_invalid && (attrs & GDA_VALUE_ATTR_DATA_NON_VALID)) {
-					GValue *sane_value;
-					GdaDataHandler *dh;
-					GType type;
-
-					check_correct_init (mgwrap);
-					dh = gnome_db_data_entry_get_handler (GNOME_DB_DATA_ENTRY (mgwrap));
-					type = gnome_db_data_entry_get_value_type (GNOME_DB_DATA_ENTRY (mgwrap));
-					sane_value = gda_data_handler_get_sane_init_value (dh, type);
-					(*mgwrap->priv->real_class->real_set_value) (mgwrap, sane_value);
+		case PROP_SET_DEFAULT_IF_INVALID: {
+			guint attrs;
+			
+			mgwrap->priv->set_default_if_invalid = g_value_get_boolean (value);
+			attrs = gnome_db_data_entry_get_attributes (GNOME_DB_DATA_ENTRY (mgwrap));
+			
+			if (mgwrap->priv->set_default_if_invalid && (attrs & GDA_VALUE_ATTR_DATA_NON_VALID)) {
+				GValue *sane_value;
+				GdaDataHandler *dh;
+				GType type;
+				
+				check_correct_init (mgwrap);
+				dh = gnome_db_data_entry_get_handler (GNOME_DB_DATA_ENTRY (mgwrap));
+				type = gnome_db_data_entry_get_value_type (GNOME_DB_DATA_ENTRY (mgwrap));
+				sane_value = gda_data_handler_get_sane_init_value (dh, type);
+				(*mgwrap->priv->real_class->real_set_value) (mgwrap, sane_value);
+				if (sane_value)
 					gda_value_free (sane_value);
-				}
 			}
 			break;
 		}
+		}
 	}
 }
 
@@ -337,6 +342,23 @@ gnome_db_entry_wrapper_contents_changed (GnomeDbEntryWrapper *mgwrap)
 	contents_changed_cb (NULL, mgwrap);
 }
 
+/**
+ * gnome_db_entry_wrapper_contents_activated
+ * @mgwrap: a #GnomeDbEntryWrapper widget
+ *
+ * Signals to @gwrap that the entry has been activated (that is the user
+ * pressed ENTER for example to signify he has finished entering data)
+ */
+void
+gnome_db_entry_wrapper_contents_activated (GnomeDbEntryWrapper *mgwrap)
+{
+	g_return_if_fail (GNOME_DB_IS_ENTRY_WRAPPER (mgwrap));
+	g_return_if_fail (((GnomeDbEntryWrapper *) mgwrap)->priv);
+
+	contents_activated_cb (NULL, mgwrap);
+}
+
+
 static void gnome_db_entry_wrapper_emit_signal (GnomeDbEntryWrapper *mgwrap);
 static void
 contents_changed_cb (GtkWidget *entry, GnomeDbEntryWrapper *mgwrap)
@@ -349,6 +371,23 @@ contents_changed_cb (GtkWidget *entry, GnomeDbEntryWrapper *mgwrap)
 	}
 }
 
+static void
+contents_activated_cb (GtkWidget *entry, GnomeDbEntryWrapper *mgwrap)
+{
+	/* @entry is not used */
+	if (! mgwrap->priv->signals_blocked) {
+		mgwrap->priv->null_forced = FALSE;
+		mgwrap->priv->default_forced = FALSE;
+#ifdef debug_signal
+		g_print (">> 'CONTENTS_ACTIVATED' from %s\n", __FUNCTION__);
+#endif
+		g_signal_emit_by_name (G_OBJECT (mgwrap), "contents_activated");
+#ifdef debug_signal
+		g_print ("<< 'CONTENTS_ACTIVATED' from %s\n", __FUNCTION__);
+#endif
+	}
+}
+
 static void
 gnome_db_entry_wrapper_emit_signal (GnomeDbEntryWrapper *mgwrap)
 {
@@ -608,8 +647,7 @@ gnome_db_entry_wrapper_set_attributes (GnomeDbDataEntry *iface, guint attrs, gui
 
 	/* Setting to DEFAULT */
 	if (mask & GDA_VALUE_ATTR_IS_DEFAULT) {
-		if ((mask & GDA_VALUE_ATTR_CAN_BE_DEFAULT) &&
-		    !(attrs & GDA_VALUE_ATTR_CAN_BE_DEFAULT))
+		if (! (mask & GDA_VALUE_ATTR_CAN_BE_DEFAULT))
 			g_return_if_reached ();
 		if (attrs & GDA_VALUE_ATTR_IS_DEFAULT) {
 			block_signals (mgwrap);
@@ -632,7 +670,6 @@ gnome_db_entry_wrapper_set_attributes (GnomeDbDataEntry *iface, guint attrs, gui
 
 			mgwrap->priv->default_forced = TRUE;
 			gnome_db_entry_wrapper_emit_signal (mgwrap);
-			return;
 		}
 		else {
 			mgwrap->priv->default_forced = FALSE;
@@ -757,13 +794,15 @@ gnome_db_entry_wrapper_get_attributes (GnomeDbDataEntry *iface)
 	}
 
 	/* has original value? */
-	if (mgwrap->priv->value_orig) {
+	if (mgwrap->priv->value_orig) 
 		retval = retval | GDA_VALUE_ATTR_HAS_VALUE_ORIG;
-	}
 	
 	if (has_current_value)
 		gda_value_free (value);
 
+	if (!mgwrap->priv->editable)
+		retval = retval | GDA_VALUE_ATTR_NO_MODIF;
+
 	return retval;
 }
 
@@ -808,8 +847,27 @@ gnome_db_entry_wrapper_set_editable (GnomeDbDataEntry *iface, gboolean editable)
 	g_return_if_fail (mgwrap->priv);
 	check_correct_init (mgwrap);
 
+	mgwrap->priv->editable = editable;
 	if (mgwrap->priv->real_class->set_editable)
 		(mgwrap->priv->real_class->set_editable) (mgwrap, editable);
 	else
 		gtk_widget_set_sensitive (GTK_WIDGET (iface), editable);
 }
+
+static void
+gnome_db_entry_wrapper_grab_focus (GnomeDbDataEntry *iface)
+{
+	GnomeDbEntryWrapper *mgwrap;
+
+	g_return_if_fail (GNOME_DB_IS_ENTRY_WRAPPER (iface));
+	mgwrap = GNOME_DB_ENTRY_WRAPPER (iface);
+	g_return_if_fail (mgwrap->priv);
+	check_correct_init (mgwrap);
+
+	if (mgwrap->priv->real_class->grab_focus)
+		(mgwrap->priv->real_class->grab_focus) (mgwrap);
+	else {
+		if (mgwrap->priv->entry && GTK_WIDGET_CAN_FOCUS (mgwrap->priv->entry))
+			gtk_widget_grab_focus (mgwrap->priv->entry);
+	}
+}
