@@ -44,6 +44,7 @@
 enum
 {
 	CONTENTS_MODIFIED,
+	CONTENTS_ACTIVATED,
 	STATUS_CHANGED,
 	LAST_SIGNAL
 };
@@ -89,6 +90,14 @@ gnome_db_data_entry_iface_init (gpointer g_class)
 				      NULL, NULL,
 				      marshal_VOID__VOID,
 				      G_TYPE_NONE, 0);
+		gnome_db_data_entry_signals[CONTENTS_ACTIVATED] =
+			g_signal_new ("contents_activated",
+				      GNOME_DB_TYPE_DATA_ENTRY,
+				      G_SIGNAL_RUN_FIRST,
+				      G_STRUCT_OFFSET (GnomeDbDataEntryIface, contents_activated),
+				      NULL, NULL,
+				      marshal_VOID__VOID,
+				      G_TYPE_NONE, 0);
 		gnome_db_data_entry_signals[STATUS_CHANGED] =
 			g_signal_new ("status_changed",
 				      GNOME_DB_TYPE_DATA_ENTRY,
@@ -113,7 +122,7 @@ gnome_db_data_entry_iface_init (gpointer g_class)
 void
 gnome_db_data_entry_set_value_type (GnomeDbDataEntry *de, GType type)
 {
-	g_return_if_fail (IS_GNOME_DB_DATA_ENTRY (de));
+	g_return_if_fail (GNOME_DB_IS_DATA_ENTRY (de));
 	g_return_if_fail (type != G_TYPE_INVALID);
 
 	if (GNOME_DB_DATA_ENTRY_GET_IFACE (de)->set_value_type)
@@ -132,7 +141,7 @@ gnome_db_data_entry_set_value_type (GnomeDbDataEntry *de, GType type)
 GType
 gnome_db_data_entry_get_value_type (GnomeDbDataEntry *de)
 {
-	g_return_val_if_fail (IS_GNOME_DB_DATA_ENTRY (de), G_TYPE_INVALID);
+	g_return_val_if_fail (GNOME_DB_IS_DATA_ENTRY (de), G_TYPE_INVALID);
 
 	if (GNOME_DB_DATA_ENTRY_GET_IFACE (de)->get_value_type)
 		return (GNOME_DB_DATA_ENTRY_GET_IFACE (de)->get_value_type) (de);
@@ -152,7 +161,7 @@ gnome_db_data_entry_get_value_type (GnomeDbDataEntry *de)
 void
 gnome_db_data_entry_set_value (GnomeDbDataEntry *de, const GValue * value)
 {
-	g_return_if_fail (IS_GNOME_DB_DATA_ENTRY (de));
+	g_return_if_fail (GNOME_DB_IS_DATA_ENTRY (de));
 
 	if (GNOME_DB_DATA_ENTRY_GET_IFACE (de)->set_value)
 		(GNOME_DB_DATA_ENTRY_GET_IFACE (de)->set_value) (de, value);
@@ -172,7 +181,7 @@ gnome_db_data_entry_set_value (GnomeDbDataEntry *de, const GValue * value)
 GValue *
 gnome_db_data_entry_get_value (GnomeDbDataEntry *de)
 {
-	g_return_val_if_fail (IS_GNOME_DB_DATA_ENTRY (de), NULL);
+	g_return_val_if_fail (GNOME_DB_IS_DATA_ENTRY (de), NULL);
 
 	if (GNOME_DB_DATA_ENTRY_GET_IFACE (de)->get_value)
 		return (GNOME_DB_DATA_ENTRY_GET_IFACE (de)->get_value) (de);
@@ -192,7 +201,7 @@ gnome_db_data_entry_get_value (GnomeDbDataEntry *de)
 void
 gnome_db_data_entry_set_value_orig (GnomeDbDataEntry *de, const GValue *value)
 {
-	g_return_if_fail (IS_GNOME_DB_DATA_ENTRY (de));
+	g_return_if_fail (GNOME_DB_IS_DATA_ENTRY (de));
 
 	if (GNOME_DB_DATA_ENTRY_GET_IFACE (de)->set_value_orig)
 		(GNOME_DB_DATA_ENTRY_GET_IFACE (de)->set_value_orig) (de, value);
@@ -208,7 +217,7 @@ void
 gnome_db_data_entry_set_current_as_orig (GnomeDbDataEntry *de)
 {
 	GValue *value;
-	g_return_if_fail (IS_GNOME_DB_DATA_ENTRY (de));
+	g_return_if_fail (GNOME_DB_IS_DATA_ENTRY (de));
 	
 	value = gnome_db_data_entry_get_value (de);
 	gnome_db_data_entry_set_value_orig (de, value);
@@ -228,7 +237,7 @@ gnome_db_data_entry_set_current_as_orig (GnomeDbDataEntry *de)
 const GValue *
 gnome_db_data_entry_get_value_orig (GnomeDbDataEntry *de)
 {
-	g_return_val_if_fail (IS_GNOME_DB_DATA_ENTRY (de), NULL);
+	g_return_val_if_fail (GNOME_DB_IS_DATA_ENTRY (de), NULL);
 
 	if (GNOME_DB_DATA_ENTRY_GET_IFACE (de)->get_value_orig)
 		return (GNOME_DB_DATA_ENTRY_GET_IFACE (de)->get_value_orig) (de);
@@ -251,7 +260,7 @@ gnome_db_data_entry_get_value_orig (GnomeDbDataEntry *de)
 void
 gnome_db_data_entry_set_value_default (GnomeDbDataEntry *de, const GValue *value)
 {
-	g_return_if_fail (IS_GNOME_DB_DATA_ENTRY (de));
+	g_return_if_fail (GNOME_DB_IS_DATA_ENTRY (de));
 	g_return_if_fail (value);
 
 	if (GNOME_DB_DATA_ENTRY_GET_IFACE (de)->set_value_default)
@@ -270,7 +279,7 @@ gnome_db_data_entry_set_value_default (GnomeDbDataEntry *de, const GValue *value
 void
 gnome_db_data_entry_set_attributes (GnomeDbDataEntry *de, guint attrs, guint mask)
 {
-	g_return_if_fail (IS_GNOME_DB_DATA_ENTRY (de));
+	g_return_if_fail (GNOME_DB_IS_DATA_ENTRY (de));
 
 	if (GNOME_DB_DATA_ENTRY_GET_IFACE (de)->set_attributes)
 		(GNOME_DB_DATA_ENTRY_GET_IFACE (de)->set_attributes) (de, attrs, mask);
@@ -287,7 +296,7 @@ gnome_db_data_entry_set_attributes (GnomeDbDataEntry *de, guint attrs, guint mas
 guint
 gnome_db_data_entry_get_attributes (GnomeDbDataEntry *de)
 {
-	g_return_val_if_fail (IS_GNOME_DB_DATA_ENTRY (de), 0);
+	g_return_val_if_fail (GNOME_DB_IS_DATA_ENTRY (de), 0);
 
 	if (GNOME_DB_DATA_ENTRY_GET_IFACE (de)->get_attributes)
 		return (GNOME_DB_DATA_ENTRY_GET_IFACE (de)->get_attributes) (de);
@@ -307,7 +316,7 @@ gnome_db_data_entry_get_attributes (GnomeDbDataEntry *de)
 GdaDataHandler  *
 gnome_db_data_entry_get_handler (GnomeDbDataEntry *de)
 {
-	g_return_val_if_fail (IS_GNOME_DB_DATA_ENTRY (de), NULL);
+	g_return_val_if_fail (GNOME_DB_IS_DATA_ENTRY (de), NULL);
 
 	if (GNOME_DB_DATA_ENTRY_GET_IFACE (de)->get_handler)
 		return (GNOME_DB_DATA_ENTRY_GET_IFACE (de)->get_handler) (de);
@@ -326,12 +335,12 @@ gnome_db_data_entry_get_handler (GnomeDbDataEntry *de)
 gboolean
 gnome_db_data_entry_expand_in_layout (GnomeDbDataEntry *de)
 {
-	g_return_val_if_fail (IS_GNOME_DB_DATA_ENTRY (de), FALSE);
+	g_return_val_if_fail (GNOME_DB_IS_DATA_ENTRY (de), FALSE);
 
 	if (GNOME_DB_DATA_ENTRY_GET_IFACE (de)->expand_in_layout)
-		(GNOME_DB_DATA_ENTRY_GET_IFACE (de)->expand_in_layout) (de);
-
-	return FALSE;
+		return (GNOME_DB_DATA_ENTRY_GET_IFACE (de)->expand_in_layout) (de);
+	else
+		return FALSE;
 }
 
 /**
@@ -344,10 +353,25 @@ gnome_db_data_entry_expand_in_layout (GnomeDbDataEntry *de)
 void
 gnome_db_data_entry_set_editable (GnomeDbDataEntry *de, gboolean editable)
 {
-	g_return_if_fail (IS_GNOME_DB_DATA_ENTRY (de));
+	g_return_if_fail (GNOME_DB_IS_DATA_ENTRY (de));
 
 	if (GNOME_DB_DATA_ENTRY_GET_IFACE (de)->set_editable)
 		(GNOME_DB_DATA_ENTRY_GET_IFACE (de)->set_editable) (de, editable);
 	else
 		gtk_widget_set_sensitive (GTK_WIDGET (de), editable);
 }
+
+/**
+ * gnome_db_data_entry_grab_focus
+ * @de: a #GtkWidget object which implements the #GnomeDbDataEntry interface
+ *
+ * Makes @de grab the focus for the window it's in
+ */
+void
+gnome_db_data_entry_grab_focus (GnomeDbDataEntry *de)
+{
+	g_return_if_fail (GNOME_DB_IS_DATA_ENTRY (de));
+
+	if (GNOME_DB_DATA_ENTRY_GET_IFACE (de)->grab_focus)
+		(GNOME_DB_DATA_ENTRY_GET_IFACE (de)->grab_focus) (de);
+}
