@@ -1,8 +1,10 @@
 /* GNOME DB library
- * Copyright (C) 1999-2002 The GNOME Foundation.
+ * Copyright (C) 1999 - 2006 The GNOME Foundation.
  *
  * AUTHORS:
  *      Rodrigo Moya <rodrigo@gnome-db.org>
+ *      Vivien Malerba <malerba@gnome-db.org>
+ *      Daniel Espinosa <esodan@gmail.com>
  *
  * This Library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Library General Public License as
@@ -21,11 +23,12 @@
  */
 
 #include <string.h>
-#include <libgnomedb/gnome-db-config.h>
+#include <libgda/gda-config.h>
 #include <libgnomedb/gnome-db-data-source-selector.h>
 #include <libgnomedb/gnome-db-util.h>
+#include <gtk/gtkcelllayout.h>
 
-#define PARENT_TYPE GTK_TYPE_OPTION_MENU
+#define PARENT_TYPE GTK_TYPE_COMBO_BOX
 
 struct _GnomeDbDataSourceSelectorPrivate {
 	guint config_listener;
@@ -36,6 +39,7 @@ static void gnome_db_data_source_selector_init       (GnomeDbDataSourceSelector
 						      GnomeDbDataSourceSelectorClass *klass);
 static void gnome_db_data_source_selector_finalize   (GObject *object);
 
+/* get a pointer to the parents to be able to call their destructor */
 static GObjectClass *parent_class = NULL;
 
 /*
@@ -46,36 +50,49 @@ static void
 show_data_sources (GnomeDbDataSourceSelector *selector, const gchar *name)
 {
 	GdaDataSourceInfo *dsn_info = NULL;
+	GtkListStore *model;
+	GtkTreeIter iter;
 
 	g_return_if_fail (GNOME_DB_IS_DATA_SOURCE_SELECTOR (selector));
 
-	gtk_option_menu_remove_menu (GTK_OPTION_MENU (selector));
+	model = gtk_list_store_new (1, G_TYPE_STRING);
 
-	if (name != NULL) {
-		dsn_info = gnome_db_config_find_data_source (name);
-		if (dsn_info != NULL) {
-			gnome_db_option_menu_add_item (GTK_OPTION_MENU (selector),
-						       dsn_info->name);
-			gnome_db_option_menu_set_selection (GTK_OPTION_MENU (selector),
-							    dsn_info->name);
-			gnome_db_config_free_data_source_info (dsn_info);
+	if (name) {
+		dsn_info = gda_config_find_data_source (name);
+				
+		if (dsn_info) {
+			
+			gtk_list_store_append (model, &iter);
+	                    
+			gtk_list_store_set (model, &iter, 0, dsn_info->name, -1);
+			
+			gtk_combo_box_set_model (GTK_COMBO_BOX (selector), GTK_TREE_MODEL (model));
+			gtk_combo_box_set_active_iter (GTK_COMBO_BOX (selector), &iter);
+			
+			gda_data_source_info_free (dsn_info);
 		}
-	} else {
+	} 
+	else {
 		GList *dsn_list, *l;
-
-		dsn_list = gnome_db_config_get_data_source_list ();
+                
+                dsn_list = gda_config_get_data_source_list ();
+		
 		for (l = dsn_list; l != NULL; l = l->next) {
 			dsn_info = (GdaDataSourceInfo *) l->data;
 			if (!dsn_info)
 				continue;
-
+                        
 			/* add the data source to the option menu */
-			gnome_db_option_menu_add_item (GTK_OPTION_MENU (selector), dsn_info->name);
+			
+			gtk_list_store_append (model, &iter);
+			
+			gtk_list_store_set (model, &iter, 0, dsn_info->name, -1);
 		}
+		
+		gtk_combo_box_set_model (GTK_COMBO_BOX(selector), GTK_TREE_MODEL(model));
+		gtk_combo_box_set_active (GTK_COMBO_BOX (selector), 0);
 
-		gnome_db_config_free_data_source_list (dsn_list);
-
-		gtk_option_menu_set_history (GTK_OPTION_MENU (selector), 0);
+		gda_config_free_data_source_list (dsn_list);
 	}
 }
 
@@ -94,16 +111,16 @@ gnome_db_data_source_selector_class_init (GnomeDbDataSourceSelectorClass *klass)
 }
 
 static void
-config_changed_cb (GConfClient *conf_client, guint cnxn_id, GConfEntry *entry, gpointer user_data)
+config_changed_cb (const gchar *path, gpointer user_data)
 {
-	const gchar *current_dsn;
+	gchar *current_dsn;
 	GnomeDbDataSourceSelector *selector = (GnomeDbDataSourceSelector *) user_data;
 
 	g_return_if_fail (GNOME_DB_IS_DATA_SOURCE_SELECTOR (selector));
 
-	current_dsn = g_strdup (gnome_db_option_menu_get_selection (GTK_OPTION_MENU (selector)));
+	current_dsn = gnome_db_data_source_selector_get_dsn (selector);
 	show_data_sources (selector, NULL);
-	gnome_db_option_menu_set_selection (GTK_OPTION_MENU (selector), current_dsn);
+	gnome_db_data_source_selector_set_dsn (selector, current_dsn);
 	g_free (current_dsn);
 }
 
@@ -111,16 +128,25 @@ static void
 gnome_db_data_source_selector_init (GnomeDbDataSourceSelector *selector,
 				    GnomeDbDataSourceSelectorClass *klass)
 {
+	GtkCellRenderer *renderer;
+
 	g_return_if_fail (GNOME_DB_IS_DATA_SOURCE_SELECTOR (selector));
 
 	/* allocate private structure */
 	selector->priv = g_new0 (GnomeDbDataSourceSelectorPrivate, 1);
 
 	/* add listener for configuration */
-	selector->priv->config_listener = gnome_db_config_add_listener (
+	selector->priv->config_listener = gda_config_add_listener (
 		"/apps/libgda/Datasources",
-		(GConfClientNotifyFunc) config_changed_cb,
+		(GdaConfigListenerFunc) config_changed_cb,
 		selector);
+
+	/* add textual renderer */
+	renderer = gtk_cell_renderer_text_new ();
+	gtk_cell_layout_pack_start (GTK_CELL_LAYOUT (selector), renderer, TRUE);
+	gtk_cell_layout_add_attribute (GTK_CELL_LAYOUT (selector), renderer,
+				       "text", 0);
+	
 }
 
 static void
@@ -130,9 +156,9 @@ gnome_db_data_source_selector_finalize (GObject *object)
 
 	g_return_if_fail (GNOME_DB_IS_DATA_SOURCE_SELECTOR (selector));
 
-	if (selector->priv->config_listener != -1) {
-		gnome_db_config_remove_listener (selector->priv->config_listener);
-		selector->priv->config_listener = -1;
+	if (selector->priv->config_listener != 0) {
+		gda_config_remove_listener (selector->priv->config_listener);
+		selector->priv->config_listener = 0;
 	}
 
 	/* free memory */
@@ -170,8 +196,8 @@ gnome_db_data_source_selector_get_type (void)
  * gnome_db_data_source_selector_new
  * @name: name of data source to display.
  *
- * Create a new #GnomeDbDataSourceSelector, which is just a #GtkOptionMenu
- * which displays, as its menu items, all the data sources currently
+ * Create a new #GnomeDbDataSourceSelector, which is just a #GtkComboBox
+ * which displays, as its items, all the data sources currently
  * configured in the system. It is useful for connection and configuration
  * screens, where the user has to choose a data source to work with.
  *
@@ -187,3 +213,52 @@ gnome_db_data_source_selector_new (const gchar *name)
 
 	return GTK_WIDGET (selector);
 }
+
+/**
+ * gnome_db_data_source_selector_get_dsn
+ * @name: name of data source to display.
+ *
+ * Get the Data Source Name (DSN) actualy selected in the #GnomeDbDataSourceSelector.
+ *
+ * Returns: the DSN name actualy selected as a new string.
+ */
+gchar *
+gnome_db_data_source_selector_get_dsn (GnomeDbDataSourceSelector *selector)
+{
+	GtkTreeIter iter;
+	gchar *dsn;
+        
+        gtk_combo_box_get_active_iter (GTK_COMBO_BOX (selector), &iter);
+        
+        gtk_tree_model_get (gtk_combo_box_get_model (GTK_COMBO_BOX(selector)), &iter, 0, &dsn, -1);
+
+	return dsn;
+}
+
+/**
+ * gnome_db_data_source_selector_set_dsn
+ * @name: name of data source to display.
+ *
+ * Set the selected Data Source Name (DSN) in the #GnomeDbDataSourceSelector.
+ *
+ */
+void
+gnome_db_data_source_selector_set_dsn (GnomeDbDataSourceSelector *selector, const gchar *dsn)
+{
+        GtkTreeIter iter;
+        GtkTreeModel *model;
+        gboolean valid;
+        gchar *actual_dsn;
+        
+        model = gtk_combo_box_get_model(GTK_COMBO_BOX(selector));
+        
+        valid = gtk_tree_model_get_iter_first (model, &iter);
+        while(valid) {
+                gtk_tree_model_get (model, &iter, 0, &actual_dsn, -1);
+                if (!g_ascii_strcasecmp (dsn, actual_dsn)) {
+                        gtk_combo_box_set_active_iter (GTK_COMBO_BOX(selector), &iter);
+                        break;
+                }
+                valid = gtk_tree_model_iter_next (model, &iter);
+        }
+}
