@@ -1,6 +1,6 @@
 /* gnome-db-sql-console.c
  *
- * Copyright (C) 2005 - 2006 Vivien Malerba
+ * Copyright (C) 2005 - 2007 Vivien Malerba
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -560,7 +560,7 @@ run_sql_cmde (GnomeDbSqlConsole *console, HistItem *hist_item, const gchar *sql)
 		}
 
 		/* run query */
-		rsql = gda_renderer_render_as_sql (GDA_RENDERER (query), dset, 0, &error);
+		rsql = gda_renderer_render_as_sql (GDA_RENDERER (query), dset, NULL, 0, &error);
 		if (rsql) {
 			GdaCommand *cmde;
 			GList *reslist;
@@ -729,7 +729,7 @@ run_internal_cmde (GnomeDbSqlConsole *console, HistItem *hist_item, const gchar
 
 		string = g_string_new (_("SQL statements:\n"
 					 "  they can span several lines, and are executed\n"
-					 "  when a ';' finished them and the ENTER key is pressed.\n"));
+					 "  when a semi-colon (;) finishes the statement and the ENTER key is pressed.\n"));
 		g_string_append (string, _("Console commands:\n"
 					   "  they start with a '\\' and are interpreted as soon\n"
 					   "  as the ENTER key is pressed. Commands are:\n"
