@@ -1,5 +1,5 @@
 /* GNOME DB library
- * Copyright (C) 1999 - 2006 The GNOME Foundation.
+ * Copyright (C) 1999 - 2007 The GNOME Foundation.
  *
  * AUTHORS:
  * 	Rodrigo Moya <rodrigo@gnome-db.org>
@@ -58,8 +58,7 @@
 #include <libgnomedb/gnome-db-data-import.h>
 #include <libgnomedb/gnome-db-data-widget-info.h>
 #include <libgnomedb/gnome-db-dbms-update-viewer.h>
-#include <libgnomedb/gnome-db-dsn-config-druid.h>
-#include <libgnomedb/gnome-db-dsn-config.h>
+#include <libgnomedb/gnome-db-dsn-editor.h>
 #include <libgnomedb/gnome-db-dsn-spec.h>
 #include <libgnomedb/gnome-db-editor.h>
 #include <libgnomedb/gnome-db-raw-form.h>
@@ -77,19 +76,8 @@
 #include <libgnomedb/gnome-db-selector.h>
 #include <libgnomedb/gnome-db-server-operation.h>
 #include <libgnomedb/gnome-db-sql-console.h>
-
-/* graphs */
-#include <libgnomedb/graph/gnome-db-canvas-cursor.h>
-#include <libgnomedb/graph/gnome-db-canvas-db-relations.h>
-#include <libgnomedb/graph/gnome-db-canvas-entity.h>
-#include <libgnomedb/graph/gnome-db-canvas-field.h>
-#include <libgnomedb/graph/gnome-db-canvas-fkconstraint.h>
-#include <libgnomedb/graph/gnome-db-canvas-item.h>
-#include <libgnomedb/graph/gnome-db-canvas-join.h>
-#include <libgnomedb/graph/gnome-db-canvas-query-struct.h>
-#include <libgnomedb/graph/gnome-db-canvas-text.h>
-#include <libgnomedb/graph/gnome-db-canvas-tip.h>
-#include <libgnomedb/graph/gnome-db-canvas.h>
+#include <libgnomedb/gnome-db-transaction-status.h>
+#include <libgnomedb/gnome-db-dsn-assistant.h>
 
 G_BEGIN_DECLS
 
